from django import forms
from django.utils.translation import ungettext, ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import force_unicode

from nano.comments import COMMENT_MAX_LENGTH
from nano.comments.models import Comment

class CommentDetailsForm(forms.Form):
    comment = forms.CharField(widget=forms.Textarea)
    part_of = forms.CharField(max_length=255, required=False, widget=forms.HiddenInput)
    back_to_url = '' #forms.UrlField()

    def check_for_duplicate_comment(self, new):
        """
        Check that a submitted comment isn't a duplicate. This might be caused
        by someone posting a comment twice. If it is a dup, silently return the *previous* comment.
        """
        possible_duplicates = Comment._default_manager.filter(
            content_type = new.content_type,
            object_pk = new.object_pk,
            comment = new.comment,
        )
        for old in possible_duplicates:
            if old.submit_date.date() == new.submit_date.date() and old.comment == new.comment:
                return old
                
        return new

class CommentForm(CommentDetailsForm):
    honeypot = forms.CharField(required=False, label=_('If you enter anything in this field your comment will be treated as spam'))

    def clean_honeypot(self):
        """Check that nothing's been entered into the honeypot."""
        value = self.cleaned_data["honeypot"]
        if value:
            raise forms.ValidationError(self.fields["honeypot"].label)
        return value



# -*- coding: utf-8 -*-
# vim: set fileencoding=utf-8 :

from math import modf, floor, ceil

from django import template
from django.template.defaultfilters import stringfilter
 
from nano.tools import grouper
 
register = template.Library()
 
@register.filter
def startswith(value, arg):
    """Usage {% if value|startswith:"arg" %}"""
    if value:
        return value.startswith(arg)
    return False
startswith.is_safe = True

@register.filter
def endswith(value, arg):
    """Usage {% if value|endswith:"arg" %}"""
    if value:
        return value.endswith(arg)
    return False
endswith.is_safe = True

@register.filter
@stringfilter
def nbr(text):
    """Replace whitespace with non-breaking-space"""

    pieces = text.split()
    text = u'\xa0'.join(pieces)
    return text.encode('utf8')
nbr.is_safe = True

@register.filter
def partition(iterable, cols=4):
    "Split an iterable into columns"

    if not iterable:
        return ()
    try:
        cols = int(cols)
    except (ValueError, TypeError):
        return None
    the_tuple = tuple(iterable)
    maxrows = int(ceil(len(the_tuple)/float(cols)))
    columns = grouper(maxrows, the_tuple)
    return zip(*tuple(columns))
partition.is_safe = True

@register.filter
def integer(text):
    "Get integer-part of float"
    _, integer = modf(float(text))
    return str(int(integer))
integer.is_safe = True

@register.filter
def fraction(text, arg=1):
    "Get fractional part of float, pad with zeroes until <arg> length"
    arg = int(arg)
    fraction, _ = modf(float(text))
    integer, fraction = str(fraction).split('.', 1)
    lf = len(fraction)
    fraction = fraction[:arg]
    if arg > lf:
        fraction = u'%s%s' % (fraction, '0'*(arg-lf))
    return fraction
fraction.is_safe = True

@register.inclusion_tag('come_back.html', takes_context=True)
def come_back(context):
    """Turn current url path into a query-part for use in urls
    
    With default template: 
    
    If the path is stored in the context as /foo/bar, tag returns '?next=/foo/bar'
    """
    whereami = context.get('whereami')
    request = context.get('request')
    if request:
        path_info = request.META.get('PATH_INFO')
    return { 'come_back': whereami or path_info or '' }



import urllib
from urlparse import urlparse

import logging
_LOG = logging.getLogger(__name__)

from django.shortcuts import render
from django.http import HttpResponseRedirect, Http404
from django.template import loader, RequestContext
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_POST
from django.core.urlresolvers import resolve
from django.views.generic import ListView
from django.utils.html import escape

from nano.comments.models import Comment
from nano.comments.forms import *

def _get_contenttype(model):
    if model:
        return ContentType.objects.get_for_model(model)
    return None

def _get_queryset(object_id=None, model=None):
    queryset = Comment.objects

    if object_id and model:
        object = model.objects.get(pk=object_id)
        contenttype = _get_contenttype(model)
        queryset = queryset.filter(object_pk=str(object.id), content_type=contenttype)
    else:
        queryset = queryset.all()
    return queryset        

def _get_object(model, object_id, object_field=None, **kwargs):
    if model and object_id:
        if object_field: # and hasattr(model, object_field):
            return model.objects.get(**{object_field: object_id})
        else:
            return model.objects.get(pk=object_id)
    return None

@login_required
def post_comment(request, object_arg='object_id', object_field=None, model=None, part_of=None, template_name='nano/comments/comment_form.html', *args, **kwargs):
    """Post a comment to the object of type ``model``, with primary key
    fetched from the field named in ``object_field``"""

    object_id = kwargs.get(object_arg, None)
    kwargs.pop(object_arg)
    assert object_id
    assert model

    contenttype = _get_contenttype(model)
    object = _get_object(model, object_id, object_field=object_field, **kwargs)
    part_of = request.REQUEST.get('part_of', None) or part_of

    cpart_of = None
    if part_of:
        try:
            cpart_of = Comment.objects.get(id=int(part_of))
        except Comment.DoesNotExist:
            pass

    good_data = {}
    good_data['content_type'] = contenttype
    good_data['object_pk'] = str(object.id)
    good_data['user'] = request.user
    good_data['part_of'] = part_of

    data = {}
    if request.method == 'POST' and contenttype and object:
        form = CommentForm(data=request.POST, initial={'part_of': part_of})
        data['commentform'] = form

        if form.is_valid():
            part_of = form.cleaned_data['part_of'] or part_of
            if part_of and not (contenttype and object):
                cpart_of = Comment.objects.get(id=int(part_of))
                good_data['content_type'] = cpart_of.content_type
                good_data['object_pk'] = str(cpart_of.content_object.pk)
            good_data['part_of'] = part_of
            good_data['comment'] = form.cleaned_data['comment']
            good_data['comment_xhtml'] = escape(good_data['comment'])

            if cpart_of or (contenttype and object):
                if request.POST.get('submit'):
                    if good_data['part_of']:
                        good_data['part_of'] = Comment.objects.get(id=int(part_of))
                    else:
                        del good_data['part_of']
                    try:
                        c = Comment(**good_data)
                    except TypeError:
                        assert False, repr(Comment)
                        raise
                    c.save()
                    return HttpResponseRedirect('../')
                else:
                    data['commentform'] = CommentForm(data=request.POST, initial=good_data)
                    data['preview'] = good_data
    else:
        form = CommentForm(initial=good_data)
        data['commentform'] = form

    return render(request, template_name, data)


class ListCommentView(ListView):
    template_name = 'nano/comments/comment_list.html'

    def get_queryset(self):
        object_arg = self.kwargs.get('object_arg', 'object_id')
        object_id = self.kwargs.get(object_arg)
        model = self.kwargs.get('model')
        object_field = self.kwargs.get('object_field', None)
        object = _get_object(model, object_id, object_field)
        queryset = _get_queryset(object_id=object.id, model=model)
        if not object_arg in ('object_id',):
            self.kwargs.pop(object_arg, None)
        return queryset
list_comments = ListCommentView.as_view()



from django.views.debug import technical_500_response
import sys

# Django snippet
# Author: zbyte64
# Posted: July 31, 2008
class UserBasedExceptionMiddleware(object):

    def process_exception(self, request, exception):
        if request.user.is_superuser:
            return technical_500_response(request, *sys.exc_info())



from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.contrib.auth.views import redirect_to_login
from django.contrib.auth.decorators import login_required
from django.template import loader, RequestContext
from django.contrib.contenttypes.models import ContentType

try:
    import json as simplejson
except:
    from django.utils import simplejson

from nano.mark.models import Mark, MarkType

@login_required
def toggle_mark(request, allow_xmlhttprequest=False):
    """
    Generic object vote function.

    The given template will be used to confirm the vote if this view is
    fetched using GET; vote registration will only be performed if this
    view is POSTed.

    If ``allow_xmlhttprequest`` is ``True`` and an XMLHttpRequest is
    detected by examining the ``HTTP_X_REQUESTED_WITH`` header, the
    ``xmlhttp_vote_on_object`` view will be used to process the
    request - this makes it trivial to implement voting via
    XMLHttpRequest with a fallback for users who don't have JavaScript
    enabled.

    Templates:``<app_label>/<model_name>_confirm_vote.html``
    Context:
        object
            The object being voted on.
        direction
            The type of vote which will be registered for the object.
    """
#     if allow_xmlhttprequest and request.is_ajax():
#         return xmlhttprequest_vote_on_object(request, model, direction,
#                                              object_id=object_id, slug=slug,
#                                              slug_field=slug_field)

    next = '/'
    if request.method == 'POST':
        model_pk = request.POST.get('model_pk', None)
        model_type = request.POST.get('model_type', None)
        
        if not (model_pk and model_type):
            return HttpResponseRedirect('/')

        ct = ContentType.objects.get(id=int(model_type))
        object = ct.get_object_for_this_type(pk=model_pk)
        #assert False, ct

        marktype = MarkType.objects.get(slug=request.POST.get('type', 'flag'))
        if not marktype:
            marktype = MarkType.objects.create(slug='flag', name='Flag', hide=True, permanent=True, verify=True)
        mark, created = Mark.objects.get_or_create(
                marked_by=request.user, 
                marktype=marktype, 
                object_pk=unicode(object.pk), 
                content_type=ct,
                )
        if not (created or mark.marktype.permanent):
            mark.delete()

        next = request.POST.get('next', None) or next

        # Django-notify-support
        if hasattr(request, 'notifications'):
            flip = {'flag': 'flagged', 'fave': 'faved'}
            request.notifications.add("You've now %s %s as %s" %
            (u'marked' if created else u'unmarked', object, flip.get(mark.marktype.slug, mark.marktype.slug)))

        if mark.marktype.verify:
            callable = 'nano_mark_verify_%s' % marktype.type
            return HttpResponseRedirect(reverse(callable))
    return HttpResponseRedirect(next)

def nano_mark_verify_flag(request, *args, **kwargs):
    template = ''
    raise Http404



from django.utils.timezone import now as tznow

from django.db import models

class QA(models.Model):
    question = models.CharField(max_length=255)
    answer = models.TextField()
    last_modified = models.DateTimeField(default=tznow, editable=False)

    class Meta:
        db_table = 'nano_faq_qa'

    def __unicode__(self):
        return self.question

    def save(self, *args, **kwargs):
        self.last_modified = tznow()
        super(QA, self).save(*args, **kwargs)



from datetime import timedelta

from django.utils.timezone import now as tznow
from django.template.loader import render_to_string

from nano.blog.models import Entry as _Entry
from nano.blog.settings import NANO_BLOG_TAGS, NANO_BLOG_SPECIAL_TAGS

_five_minutes = timedelta(seconds=5*60)

def add_entry_to_blog(obj, headline, template, date_field='last_modified'):
    """Auto-blog about <obj>

    obj: object to blog about

    headline: string w/o html
    
    template: template, given the context 'obj' mapping to the object
    
    date_field: date_field on object to be used for publishing date
    """
    data = {'obj': obj}
    current_time = tznow()
    template = render_to_string(template, dictionary=data)
    pub_date = obj.__dict__.get(date_field, current_time)
    latest = _Entry.objects.latest()
    # Prevent duplicates
    if not (latest.headline == headline and latest.pub_date > current_time - _five_minutes):
        blog_entry = _Entry.objects.create(content=template,headline=headline,pub_date=pub_date)
        return blog_entry

def get_nano_blog_entries(special_tags=NANO_BLOG_SPECIAL_TAGS, cutoff=2):
    """Fetch <cutoff> number of most recent blog entries and split out
    entries tagged with <special_tags>.

    cutoff: integer, default 2
    
    special_tags: iterable collection of strings, default NANO_BLOG_SPECIAL_TAGS
    """
    special_news = ()
    news = _Entry.objects
    if NANO_BLOG_TAGS:
        if NANO_BLOG_TAGS.__name__ == 'taggit':
            special_news = news.filter(tags__slug__in=special_tags)
        elif NANO_BLOG_TAGS.__name__ == 'tagging':
            special_news = news.tagged.with_any(special_tags)
        special_news = special_news.order_by('-pub_date')[:cutoff]
        if special_news:
            news = news.exclude(id__in=[e.id for e in special_news])
        else:
            news = news.all()
    news = news.order_by('-pub_date')[:cutoff]
    return news, special_news



from django.conf.urls import *

urlpatterns = patterns('nano.blog.views',
        (r'^(?P<year>\d{4})/(?P<month>[01]\d)/(?P<day>[0123]\d)/$', 'list_entries_by_date'),
        (r'^(?P<year>\d{4})/(?P<month>[01]\d)/$', 'list_entries_by_year_and_month'),
        (r'^(?P<year>\d{4})/$',     'list_entries_by_year'),
        (r'^latest/$',              'list_latest_entries'),
        (r'^today/$',               'list_entries_for_today'),
        (r'^$',                     'list_entries'),
)



import logging
import unicodedata
from itertools import izip_longest

_LOG = logging.getLogger(__name__)

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import render
from django.template import RequestContext, loader, Context
from django.db.models import get_model

class SiteProfileNotAvailable(Exception):
    pass

def nullfunction(return_this=None, *args, **kwargs):
    return return_this

def pop_error(request):
    error = request.session.get('error', None)
    if 'error' in request.session:
        del request.session['error']
    return error

def asciify(string):
    """Convert unicode string to ascii, normalizing with NFKD"""

    string = unicodedata.normalize('NFKD', string)
    return string.encode('ascii', 'ignore')

def render_page(request, *args, **kwargs):
    return render(request, *args, **kwargs)

def grouper(n, iterable, fillvalue=None):
    "grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx"
    args = [iter(iterable)] * n
    return izip_longest(fillvalue=fillvalue, *args)

def get_profile_model(raise_on_error=True):
    if not getattr(settings, 'AUTH_PROFILE_MODULE', False):
        error = "AUTH_PROFILE_MODULE isn't set in the settings, couldn't fetch profile"
        _LOG.error(error)
        if raise_on_error:
            raise SiteProfileNotAvailable(error)
        return None
    try:
        app_label, model_name = settings.AUTH_PROFILE_MODULE.split('.')
        model = get_model(app_label, model_name)
    except ImportError:
        error = "Could not import the profile '%s' given in AUTH_PROFILE_MODULE" % settings.AUTH_PROFILE_MODULE
        _LOG.error(error)
        if raise_on_error:
            raise SiteProfileNotAvailable(error)
        return None
    except ImproperlyConfigured:
        error = "An unknown error happened while fetching the profile model"
        if raise_on_error:
            raise SiteProfileNotAvailable(error)
        return None
    return model

def get_user_model():
    app_label, model_name = getattr(settings, 'NANO_USER_MODEL', 'auth.User').split('.')
    return get_model(app_label, model_name)

if 'nano.blog' in settings.INSTALLED_APPS:
    try:
        from nano.blog import add_entry_to_blog
    except ImportError:
        add_entry_to_blog = nullfunction
else:
    add_entry_to_blog = None



from django.views.generic import ListView

from nano.faq.models import QA


class ListFAQView(ListView):
    queryset = QA.objects.all().order_by('question')

    def get_context_data(self, **kwargs):
        context = super(ListFAQView, self).get_context_data(**kwargs)
        context['me'] = 'faq'
        return context
list_faqs = ListFAQView.as_view()



from django.views.generic import DetailView, ListView

from nano.badge.models import Badge


class BadgeMixin(object):
    queryset = Badge.objects.all()

    def get_context_data(self, **kwargs):
        context = super(BadgeMixin, self).get_context_data(**kwargs)
        context['me'] = 'badge'
        return context


class ListBadgeView(BadgeMixin, ListView):
    pass
list_badges = ListBadgeView.as_view()


class DetailBadgeView(BadgeMixin, DetailView):
    pass
show_badge = DetailBadgeView.as_view()



from django.conf.urls import *

from nano.comments.models import *

comment_dict = {}


urlpatterns = patterns('nano.comments.views',
    (r'^$',         'list_comments', comment_dict, 'comments-list-comments'),
    (r'^post$',     'post_comment', {}, 'comments-post-comment'),
)

urlpatterns += patterns('',
    url(r'^cr/(\d+)/(.+)/$', 'django.views.defaults.shortcut', name='comments-url-redirect'),
)




from django.contrib import admin

from nano.privmsg.models import PM

class PMAdmin(admin.ModelAdmin): 
    model = PM
    ordering = ('subject', '-sent')
    list_display = ('subject', 'sender', 'recipient', 'sent', 'recipient_archived', 'recipient_deleted', 'sender_deleted')
    list_filter = ('sender', 'recipient',)
    search_fields = ('sender', 'recipient', 'subject', 'sent')
admin.site.register(PM, PMAdmin)





from django.contrib import admin

from nano.blog.models import *
from nano.blog.settings import TAGGING, TAGGIT, NANO_BLOG_TAGS

tag_devel = None
untag_devel = None
if NANO_BLOG_TAGS:

    if NANO_BLOG_TAGS == TAGGIT:

        def tag_devel(modeladmin, request, queryset):
            for entry in queryset:
                entry.tags.add('devel')

        def untag_devel(modeladmin, request, queryset):
            for entry in queryset:
                entry.tags.remove('devel')

    elif NANO_BLOG_TAGS == TAGGING:
        Tag = TAGGING.models.Tag

        def tag_devel(modeladmin, request, queryset):
            for entry in queryset:
                Tag.objects.add_tag(entry, 'devel')

        def untag_devel(modeladmin, request, queryset):
            for entry in queryset:
                start, _, end = entry.tags.partition('devel')
                if not end:
                    continue
                entry.tags = '%s %s' % (start.strip(), end.strip())
                entry.save()

    tag_devel.short_description = "Tag selected entries with 'devel'"
    untag_devel.short_description = "Remove 'devel'-tag from selected entries"

class EntryAdmin(admin.ModelAdmin):
    model = Entry
    list_display = ('headline', 'pub_date')
    search_fields = ('headline', 'pub_date')
    date_hierarchy = 'pub_date'
    if tag_devel and untag_devel:
        actions = [tag_devel, untag_devel]

admin.site.register(Entry, EntryAdmin)



import datetime

from django.views.generic import ListView
from django.views.generic import (ArchiveIndexView,
                                  YearArchiveView,
                                  MonthArchiveView,
                                  DayArchiveView,
                                  TodayArchiveView,
)

from nano.blog.models import Entry

class BlogMixin(object):
    queryset = Entry.objects.all().order_by('-pub_date')

    def get_context_data(self, **kwargs):
        context = super(BlogMixin, self).get_context_data(**kwargs)
        context['me'] = 'news'
        context['now_today'] = datetime.date.today()
        return context

class BlogDateMixin(BlogMixin):
    date_field = 'pub_date'

class MonthBlogMixin(BlogDateMixin):
    allow_empty = True
    month_format = '%m'

class ListBlogView(BlogMixin, ListView):
    pass
list_entries = ListBlogView.as_view()

class YearBlogView(BlogDateMixin, YearArchiveView):
    allow_empty = True
    make_object_list = True
list_entries_by_year = YearBlogView.as_view()

class MonthBlogView(MonthBlogMixin, MonthArchiveView):
    pass
list_entries_by_year_and_month = MonthBlogView.as_view()

class DayBlogView(MonthBlogMixin, MonthArchiveView):
    pass
list_entries_by_date = DayBlogView.as_view()

class TodayBlogView(MonthBlogMixin, MonthArchiveView):
    pass
list_entries_for_today = TodayBlogView.as_view()

class LatestBlogView(BlogDateMixin, ArchiveIndexView):
    context_object_name = 'object_list'
list_latest_entries = LatestBlogView.as_view()



from django.db import models

class Chunk(models.Model):
    slug = models.SlugField()
    content = models.TextField()

    class Meta:
        db_table = 'nano_chunk_chunk'

    def __unicode__(self):
        return self.slug
    



from django.contrib import admin

from nano.mark.models import MarkType, Mark

class MarkAdmin(admin.ModelAdmin):
    model = Mark
    ordering = ('marked_at',)
    list_display = ('marktype', 'marked_by')
    list_filter = ('marktype',)
admin.site.register(Mark, MarkAdmin)

class MarkTypeAdmin(admin.ModelAdmin):
    model = MarkType
    ordering = ('name',)
    list_display = ('name',)
    list_filter = ('verify','permanent','hide')
admin.site.register(MarkType, MarkTypeAdmin)




from django import forms

from nano.privmsg.models import *

class PMForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(PMForm, self).__init__(*args, **kwargs)
        self.fields['text'].widget = forms.Textarea(attrs={'cols': '80'})
        
    class Meta:
        model = PM
        fields = ('text', 'subject')




from math import ceil

from django import template
from django.core.urlresolvers import reverse

from nano.badge.models import Badge
from nano.tools import grouper

register = template.Library()

# \u25cf BLACK CIRCLE
# \u26ab MEDIUM BLACK CIRCLE (not as common)
SYMBOLS = {
    100: u'\u25cf',
    200: u'\u25cf',
    300: u'\u25cf',
}

SYMBOL_NAMES = {
    100: 'bronze',
    200: 'silver',
    300: 'gold',
}

def sum_badges(user):
    levels = {}
    for badge in user.badges.all():
        levels[badge.level] = levels.setdefault(badge.level, 0) + 1

    return levels

def get_badges_for_user(user):
    inner_template = u'<span class="b%i" title="%s %s badge%s">%s</span>%i'
    levels = sum_badges(user)
    sorted_levels = reversed(sorted(levels.keys()))
    badge_list = []
    for level in sorted_levels:
        name = SYMBOL_NAMES[level]
        symbol = SYMBOLS[level]
        num_levels = levels[level]
        plural = u's' if num_levels > 1 else u''
        badge_list.append(inner_template % (level, num_levels, name, plural, symbol, num_levels))
    return badge_list

@register.simple_tag
def show_badges(user):
    outer_template = u'<span>%s</span>'

    badge_list = get_badges_for_user(user)

    if badge_list:
        return outer_template % u'\xa0'.join(badge_list)
    return u''

@register.simple_tag
def show_badges_as_table(user, cols=4):
    outer_template = u'<table>%s</table>'
    cell = u'<td>%s</td>'
    row = u'<tr>%s</tr>\n'
    single_col = u'<tr><td>%s</td></tr>\n'

    badge_list = get_badges_for_user(user)

    if cols == 1:
        return [single_col % badge for badge in badge_list]
    elif cols > 1:
        piecesize = int(ceil(len(badge_list) / float(cols)))
        badge_lists = grouper(piecesize, badge_list)
        outer = []
        go_over = list(range(cols))
        for p in range(piecesize):
            inner = []
            for i in go_over:
                inner.append(cell % badge_list[i][p])
            outer.append(row % u''.join(inner))

    return outer_template % u''.join(outer)

@register.simple_tag
def show_badge(badge):
    if not badge: return u''
    template = u'<span class="badge"><a href="%(link)s"><span class="b%(level)i" >%(symbol)s</span> %(name)s</a></span>'
    fillin = {
        'level': badge.level,
        'symbol': SYMBOLS[badge.level],
        'name': badge.name,
        'link': reverse('badge-detail', args=[badge.id]),
    }
    return template % fillin

@register.simple_tag
def show_badge_and_freq(badge):
    template = u'<span class="badge-freq">%s (%i)</span>'
    badge_text = show_badge(badge)
    return template % (badge_text, badge.receivers.count())



from django.conf.urls import *

signup_done_data = { 'template_name': 'nano/user/signup_done.html' }

# 'project_name' should be a setting
password_reset_data = {'project_name': 'CALS'}

urlpatterns = patterns('nano.user.views',
    (r'^signup/$',          'signup', {}, 'nano_user_signup'),
    (r'^password/reset/$',  'password_reset', password_reset_data, 'nano_user_password_reset'),
    (r'^password/change/$', 'password_change', {}, 'nano_user_password_change'),
)

urlpatterns += patterns('',
    url(r'^signup/done/$', 'django.shortcuts.render', signup_done_data, name='nano_user_signup_done'),
)



from django.contrib import admin
from nano.chunk.models import Chunk
 
class ChunkAdmin(admin.ModelAdmin):
    ordering = ['slug',]
    list_display = ('slug',)
    search_fields = ('slug', 'content')

admin.site.register(Chunk, ChunkAdmin)



from django.contrib import admin

from nano.badge.models import Badge

class BadgeAdmin(admin.ModelAdmin):
    list_display = ('level', 'name', 'description')
    list_filter = ('level',)
    ordering = ('level', 'name',)
admin.site.register(Badge, BadgeAdmin)




from django import forms
from django.db import models

class SignupForm(forms.Form):
    help_text = {
            'last_name': """If you set "Personal name" or """
                    """"Family name", these will """
                    """be shown instead of the username""",
            'email': """Used to mail you your password, """
                    """should you forget it""",
            'tt1': """If you're human, leave this field empty""",
            'username': """Preferrably ASCII, without punctuation and spaces, and lowercase""",
    }
    username = forms.CharField(label='Username', max_length=30,
            min_length=2, 
            help_text=help_text['username'])
    password1 = forms.CharField(label='Password', max_length=30, widget=forms.PasswordInput)
    password2 = forms.CharField(label='Repeat password', max_length=30, widget=forms.PasswordInput)
    email = forms.EmailField(label='Email', required=True, help_text=help_text['email'])
    turing_test1 = forms.CharField(label='Turing test',
            required=False, max_length=10, help_text=help_text['tt1'])

    def clean_password2(self):
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError("The two password fields didn't match.")
        return password2

    def clean_turing_test1(self):
        tt = self.cleaned_data.get('turing_test1').strip()
        if tt:
            raise forms.ValidationError("You failed the Turing Test")
        return tt

class PasswordChangeForm(forms.Form):
    password1 = forms.CharField(label='New password', max_length=30, widget=forms.PasswordInput)
    password2 = forms.CharField(label='Repeat new password', max_length=30, widget=forms.PasswordInput)

    def clean_password2(self):
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return password2

class PasswordResetForm(forms.Form):
    username = forms.CharField(label='Username', max_length=30, min_length=2)
    secret = forms.CharField(label='Secret', max_length=64, required=False)



from django.template import Library, Context
from django.template.loader import get_template
from django.contrib.contenttypes.models import ContentType

from nano.mark.models import MarkType, Mark

register = Library()

@register.inclusion_tag('nano/mark/mark.html', takes_context=True)
def mark(context, model, type="flag"):
    ok_type = MarkType.objects.filter(slug=type)
    model_pk = model.pk
    ct = ContentType.objects.get_for_model(model)
    model_type = ct.id

    status = Mark.objects.filter(object_pk=str(model_pk), content_type=ct).count()

    request = context['request']
    user = request.user
    next = request.path
    return {
        'model_pk': model_pk,
        'model_type': model_type,
        'type': type if ok_type else "flag",
        'unique_id': '%s_%s' % (type, model_pk),
        'user': user,
        'next': next,
        'status': status,
    }

@register.inclusion_tag('nano/mark/mark_faved.html', takes_context=True)
def mark_faved(context, model):
    return mark(context, model, 'fave')



from django.utils.timezone import now as tznow
from django.template.defaultfilters import slugify
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _
#from django.conf import settings

from nano.tools.models import GenericForeignKeyAbstractModel

from nano.mark.managers import MarksManager

class MarkedMixin(models.Model):
    "Used by marked models"

    class Meta:
        abstract = True

    def marks(self):
        ct = ContentType.objects.get_for_model(self)
        return Mark.objects.filter(content_type = ct, object_pk = unicode(self.pk))
    
    def flagged(self):
        return self.marks.filter(marktype__slug='flag')

    def faved(self):
        return self.marks.filter(marktype__slug='fave')

    def scrambled(self):
        return self.marks.filter(marktype__slug='scrambled')

    def removed(self):
        return self.marks.filter(marktype__slug='removed')

    @property
    def hidden(self):
        marks = self.marks.filter(marktype__slug='flag')
        return True if marks.filter(marktype__hide=True) else False

class MarkType(models.Model):
    name = models.CharField(max_length=32)
    slug = models.SlugField(unique=True)
    hide = models.BooleanField(default=False)
    verify = models.BooleanField(default=False)
    permanent = models.BooleanField(default=False)

    class Meta:
        db_table='nano_mark_marktype'

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        self.slug = slugify(self.name)
        super(MarkType, self).save(*args, **kwargs)

class Mark(GenericForeignKeyAbstractModel):
    marked_by = models.ForeignKey(User, verbose_name=_('user'), related_name="marks") 
    marked_at = models.DateTimeField(_('date/time marked'), default=tznow)
    marktype = models.ForeignKey(MarkType)
    comment = models.CharField(max_length=256, blank=True, null=True)
    
    objects = MarksManager()

    class Meta:
        db_table='nano_mark_mark'

    class Meta:
        db_table = "nano_mark"
        ordering = ('marked_at',)
        get_latest_by = 'marked_at'

    def __unicode__(self):
        return "%s have marked %s" % (self.marked_by, self.content_object)    

    def save(self, parent=None, *args, **kwargs):
        if self.marked_at is None:
            self.marked_at = tznow()
        super(Mark, self).save(*args, **kwargs)




from django.db import models
from django.contrib.auth.models import User

class DefaultManager(models.Manager):
    pass

class BadgeRecipientManager(models.Manager):

    def get_all_recipients(self):
        return User.objects.filter(badges__isnull=False).distinct()

    def get_all_nonrecipients(self):
        return User.objects.exclude(badges__isnull=False)

class Badge(models.Model):
    """
    Three fields:
        level - integer, default: 100
        name - text, max. 20 chars
        description - text, aim for one line

    receivers -> User.badges
    """
    level = models.PositiveIntegerField(default=100)
    name = models.CharField(max_length=20, unique=True)
    description = models.TextField()
    receivers = models.ManyToManyField(User, blank=True, null=True, related_name='badges')

    objects = BadgeRecipientManager()

    class Meta:
        db_table = 'nano_badge_badge'

    def __unicode__(self):
        return self.name





def add_badge(badge, model):
    "Put a badge on a model"
    if badge not in model.badges.all():
        model.badges.add(badge)

def batchbadge(badge, queryset):
    "Put a badge on all models that do not already have the badge"
    for model in queryset:
        add_badge(badge, model)



from django.db import models
from nano.blog.settings import TAGGING, TAGGIT, NANO_BLOG_TAGS

# Optional support for django-taggit
# For some unfathomable reason this cannot be imported inside the class
# as of Django 1.3
if NANO_BLOG_TAGS and NANO_BLOG_TAGS == TAGGIT:
    from taggit.managers import TaggableManager

class Entry(models.Model):
    headline = models.CharField(max_length=255)
    content = models.TextField()
    pub_date = models.DateTimeField()

    # Optional support for django-taggit
    if NANO_BLOG_TAGS and NANO_BLOG_TAGS == TAGGIT:
        tags = TaggableManager(blank=True)

    class Meta:
        db_table = 'nano_blog_entry'
        verbose_name_plural = 'entries'
        ordering = ('-pub_date',)
        get_latest_by = 'pub_date'

    def __unicode__(self):
        return self.headline

# Optional support for django-tagging
if NANO_BLOG_TAGS and NANO_BLOG_TAGS == TAGGING:
    TAGGING.register(Entry)



from django.conf.urls import *

urlpatterns = patterns('nano.activation.views',
    url(r'^activate$',         'activate_key', name='nano-activate-key'),
)

urlpatterns += patterns('',
    url(r'^activation_ok/$', 'django.shortcuts.render', 
                             {'template_name': 'nano/activation/activated.html'},
                             name='nano-activation-ok'),
)




from django.conf.urls import *

urlpatterns = patterns('nano.badge.views',
        url(r'^$',                        'list_badges', {}, 'badge-list'),
        url(r'^(?P<pk>[0-9]+)/$',  'show_badge', {}, 'badge-detail'),
)



import django.dispatch
from django.db import models
from django.contrib.auth.models import User
from django.utils.timezone import now as tznow

from nano.activation.signals import key_activated

Q = models.Q

class ActivationError(Exception):
    pass

class ActivationKeyError(ActivationError):
    pass

def activate(keystring, user):
    """Activates a specific key for user, returns True on activation,
    raises an exception otherwise"""
    now = tznow()
    try:
        key = Key.objects.get(key=keystring)
    except Key.DoesNotExist:
        raise ActivationKeyError('Key %s does not exist, typo?' % keystring)
    if key.expires and key.expires <= now:
        raise ActivationKeyError('Key expired on %s' % key.expires)
    if key.activated:
        raise ActivationKeyError('Key has already been activated')
    key.activated_by = user
    key.activated = now
    key.save()
    key_activated.send_robust(sender=key, user=user, group=key.group)

class KeyManager(models.Manager):

    def expired(self):
        now = tznow()
        return self.get_queryset().exclude(expires=None).filter(expires__lte=now)

    def available(self):
        now = tznow()
        return self.get_queryset().filter(Q(expires__gt=now)|Q(expires=None)).filter(activated=None)

    def activated(self):
        now = tznow()
        return self.get_queryset().exclude(activated=None)

    def activate(self, *args):
        return activate(*args)

class Key(models.Model):
    key = models.CharField(max_length=255)
    group = models.SlugField(max_length=32, blank=True, null=True)
    pub_date = models.DateTimeField(auto_now_add=True)
    expires = models.DateTimeField(blank=True, null=True)
    activated = models.DateTimeField(blank=True, null=True)
    activated_by = models.ForeignKey(User, blank=True, null=True, related_name='activation_keys')

    objects = KeyManager()

    class Meta:
        db_table = 'nano_activation_code'
        ordering = ('-pub_date',)
        get_latest_by = 'pub_date'

    def __unicode__(self):
        pp_pub_date = self.pub_date
        pp_expires = self.expires or ''
        return u"%s (%s) %s %s" % (self.key, self.group, pp_pub_date, pp_expires)

    def activate(self, user):
        return activate(self.key, user)



from django.utils.timezone import now as tznow
from django.conf import settings
from django.db import models
from django.contrib.auth.models import User

from nano.tools.models import UnorderedTreeMixin, GenericForeignKeyAbstractModel

COMMENT_MAX_LENGTH = getattr(settings,'COMMENT_MAX_LENGTH',3000)

class Comment(GenericForeignKeyAbstractModel, UnorderedTreeMixin):
    user = models.ForeignKey(User,
            blank=True, null=True, related_name="%(class)s_comments") 
    comment = models.TextField(max_length=COMMENT_MAX_LENGTH)
    comment_xhtml = models.TextField(editable=False)
    added = models.DateTimeField(default=tznow)
    is_visible = models.BooleanField(default=True)
    is_scrambled = models.BooleanField(default=False)

    objects = models.Manager()

    class Meta:
        db_table = "nano_comments_comment"
        ordering = ('added',)
        get_latest_by = 'added'

    def __unicode__(self):
        return "%s: %s..." % (self.user, self.comment[:49]+'...' if len(self.comment) > 50 else self.comment)    
        
    def get_content_object_url(self):
        return self.content_object.get_absolute_url() or ''

    def get_absolute_url(self, anchor_pattern="#c%(id)s"):
        content_url = self.get_content_object_url()
        anchor = anchor_pattern % self.__dict__
        if content_url:
            return content_url + anchor
        return anchor

    def roots(self):
        tree = self._default_manager.filter(part_of__isnull=True)
        return tree.filter(content_type=self.content_type, object_pk=self.object_pk)

    def get_path(self):
        return [self._default_manager.get(id=p).filter(content_type=self.content_type, object_pk=self.object_pk) 
                for p in unicode(self.path).split(self._sep) if p]

    def descendants(self):
        tree = self._default_manager.filter(path__startswith=self.path).exclude(id=self.id)
        return tree.filter(content_type=self.content_type, object_pk=self.object_pk)




"""
Wrapper for loading templates from the filesystem.
"""

from django.conf import settings
from django.template.base import TemplateDoesNotExist
from django.template.loader import BaseLoader
from django.utils._os import safe_join

from django.db import models

Chunk = models.get_model('chunk', 'Chunk')

class Loader(BaseLoader):
    is_usable = True

    def load_template_source(self, template_name, template_dirs=None):
        chunk_model_name = Chunk.__name__
        try:
            chunk = Chunk.objects.get(slug=template_name)
            return (chunk.content, "chunk:%s:%s" % (chunk_model_name, template_name))
        except Chunk.DoesNotExist:
            error_msg = "Couldn't find a %s-chunk with the name %s" % (chunk_model_name, template_name)
            raise TemplateDoesNotExist(error_msg)

_loader = Loader()




from django.contrib import admin

from nano.faq.models import *

class QAAdmin(admin.ModelAdmin):
    model = QA
    list_display = ('question', 'answer', 'last_modified')

admin.site.register(QA, QAAdmin)



from django.contrib import admin

from nano.blog.models import *
from nano.activation.models import Key

class KeyAdmin(admin.ModelAdmin):
    model = Key
    list_display = ('key', 'group', 'pub_date', 'activated_by', 'expires')
    list_filter = ('group', 'activated_by')
    date_hierarchy = 'pub_date'

admin.site.register(Key, KeyAdmin)



from django.db import models
from django.db.models.query import QuerySet
from django.contrib.contenttypes.models import ContentType

class MarkedManager(models.Manager):
    "Extra manager for objects that are marked"

    def flagged(self):
        ct = ContentType.objects.get_for_model(self.model)
        return super(MarkedManager, self).get_queryset().filter(author='Roald Dahl')

    def favorited(self):
        ct = ContentType.objects.get_for_model(self.model)
        return super(MarkedManager, self).get_queryset().filter(author='Roald Dahl')

class MarksQuerySet(QuerySet):
    "Adds Mark-specific filters"

    def flagged(self):
        return self.filter(marktype__slug='flag')

    def faved(self):
        return self.filter(marktype__slug='fave')

    def scrambled(self):
        return self.filter(marktype__slug='scrambled')

    def removed(self):
        return self.filter(marktype__slug='removed')

class MarksManager(models.Manager):
    "Needed to use the custom queryset"
    
    use_for_related_fields = True

    def get_queryset(self):
        return MarksQuerySet(self.model)

    def flagged(self):
        return self.get_queryset().flagged()

    def faved(self):
        return self.get_queryset().faved()

    def scrambled(self):
        return self.get_queryset().scrambled()

    def removed(self, orgs):
        return self.get_queryset().removed()




from django.conf.urls import *

urlpatterns = patterns('nano.privmsg.views',
    url(r'^add$',          'add_pm', name='add_pm'),
    url(r'^(?P<msgid>[1-9][0-9]*)/archive$', 'move_to_archive', name='archive_pm'),
    url(r'^(?P<msgid>[1-9][0-9]*)/delete$', 'delete', name='delete_pm'),
    #url(r'^(?:(?P<action>(archive|sent))/?)?$', 'show_pms', name='show_pms'),
    url(r'^archive/$', 'show_pm_archived', name='show_archived_pms'),
    url(r'^sent/$', 'show_pm_sent', name='show_sent_pms'),
    url(r'^$', 'show_pm_received', name='show_pms'),
    #url(r'^$', 'show_pms', {u'action': u'received'}, name='show_pms'),
)




__all__ = ('Country',)

from django.db import models

class Country(models.Model):
    iso = models.CharField(max_length=2, primary_key=True)
    name = models.CharField('Country name', max_length=128)

    class Meta:
        verbose_name = 'country'
        verbose_name_plural = 'countries'
        db_table = 'nano_country'

    @property
    def printable_name(self):
        return self.name

    def __unicode__(self):
        return self.name



from random import choice, sample
import string

from django.utils.timezone import now as tznow
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.template.defaultfilters import slugify

from nano.tools import pop_error, get_profile_model, asciify
from nano.user.forms import SignupForm, PasswordChangeForm, PasswordResetForm
from nano.user import new_user_created


import logging
_LOG = logging.getLogger(__name__)

class NanoUserError(Exception):
    pass

class NanoUserExistsError(NanoUserError):
    pass

# def pop_error(request):
#     error = request.session.get('error', None)
#     if 'error' in request.session:
#         del request.session['error']
#     return error

def random_password():
    sample_space = string.letters + string.digits + r'!#$%&()*+,-.:;=?_'
    outlist = []
    for i in xrange(1,8):
        chars = sample(sample_space, 2)
        outlist.extend(chars)
    return ''.join(outlist)

def make_user(username, password, email=None, request=None):
    try:
        User.objects.get(username=username)
    except User.DoesNotExist:
        # make user
        user = User(username=username[:30])
        user.set_password(password)
        user.is_staff = False
        user.is_superuser = False
        user.is_active = True
        if email:
            user.email = email
        user.save()
        
        # Create profile
        Profile = get_profile_model(raise_on_error=False)
        if Profile:
            profile = Profile(user=user, display_name=username)
            profile.save()

        # Don't signal creation of test users
        test_users = getattr(settings, 'NANO_USER_TEST_USERS', ())
        for test_user in test_users:
            if user.username.startswith(test_user):
                break
        else:
            new_user_created.send(sender=User, user=user) 
        if request is not None:
            infomsg = u"You're now registered, as '%s'" % username
            messages.info(request, infomsg)
            _LOG.debug('Created user: %s/%s' % (user, user.check_password(password)))
        return user
    else:
        raise NanoUserExistsError("The username '%s' is already in use by somebody else" % username)

def signup(request, template_name='signup.html', *args, **kwargs):
    me = 'people'
    error = pop_error(request)
    data = {
            'me': me, 
            'error': error, 
            'form': SignupForm()
    }
    if request.method == 'POST':
        form = SignupForm(data=request.POST)
        if form.is_valid():
            username = asciify(form.cleaned_data['username'])
            password = form.cleaned_data['password2']
            email = form.cleaned_data['email'].strip() or ''

            errormsg = u'Username "%s" is taken'

            # check that username not taken
            userslug = slugify(username)
            Profile = get_profile_model(raise_on_error=False)
            if Profile.objects.filter(slug=userslug).count():
                # error!
                safe_username = slugify('%s-%s' % (username, str(tznow())))
                changed_warningmsg = errormsg + ", changed it to '%s'."
                messages.warning(request, changed_warningmsg % (username, safe_username))
                username = safe_username

            # make user
            try:
                user = make_user(username, password, email=email, request=request)
            except NanoUserExistsError:
                next_profile = Profile.objects.get(user=user).get_absolute_url()
                return HttpResponseRedirect(next_profile)
            else:
                # fake authentication, avoid a db-lookup/thread-trouble/
                # race conditions
                user.backend = 'django.contrib.auth.backends.ModelBackend'
                _LOG.debug('Attempting login of: %s' % user)
                login(request, user)
                nexthop = getattr(settings, 'NANO_USER_SIGNUP_NEXT', reverse('nano_user_signup_done'))
                try:
                    nexthop_profile = Profile.objects.get(user=user).get_absolute_url()
                    return HttpResponseRedirect(nexthop_profile)
                except Profile.DoesNotExist:
                    pass
                return HttpResponseRedirect(nexthop)
            _LOG.debug('Should never end up here')
    return render(request, template_name, data)

@login_required
def password_change(request, *args, **kwargs):
    error = pop_error(request)
    template_name = 'password_change_form.html'
    if request.method == "POST":
        form = PasswordChangeForm(request.POST)
        if form.is_valid():
            password = form.cleaned_data[u'password2']
            user = request.user
            user.set_password(password)
            user.save()
            request.session['error'] = None
            return HttpResponseRedirect('/password/change/done/')
    else:
        form = PasswordChangeForm()
    data = { 'form': form,
            'error': error,}
    return render(request, template_name, data)

def password_reset(request, project_name='Nano', *args, **kwargs):
    error = pop_error(request)
    template = 'password_reset_form.html'
    e_template = 'password_reset.txt'
    help_message = None
    e_subject = '%s password assistance' % project_name
    e_message = """Your new password is: 

%%s

It is long deliberately, so change it to 
something you'll be able to remember.


%s' little password-bot
""" % project_name
    e_from = getattr(settings, 'NANO_USER_EMAIL_SENDER', '')
    form = PasswordResetForm()
    if e_from and request.method == 'POST':
        form = PasswordResetForm(request.POST)
        if form.is_valid():
            user = get_object_or_404(User, username=form.cleaned_data['username'])
            if user.email:
                tmp_pwd = random_password()
                user.set_password(tmp_pwd)
                result = send_mail(subject=e_subject, from_email=e_from, message=e_message % tmp_pwd, recipient_list=(user.email,))
                user.save()
                request.session['error'] = None
                return HttpResponseRedirect('/password/reset/sent/')
            else:
                error = """There's no email-address registered for '%s', 
                        the password can't be reset.""" % user.username
                request.session['error'] = error
                
    data = {'form': form,
            'help_message': help_message,
            'error':error}
    return render(request, template, data)




"""
Mixin-models, with minimal example implementations.

"""

from django.conf import settings
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.core import urlresolvers
from django.db import models
from django.utils.translation import ugettext_lazy as _

class UnorderedTreeManager(models.Manager):
    def roots(self):
        "Return a list of tree roots, nodes having no parents"
        return self.get_queryset().filter(part_of__isnull=True)

class UnorderedTreeMixin(models.Model):
    part_of = models.ForeignKey('self', blank=True, null=True, default=None, related_name='has_%(class)s_children')
    path = models.CharField(max_length=255, blank=True, default='')

    tree = UnorderedTreeManager()

    _sep = u'/'

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if not self.id:
            super(UnorderedTreeMixin, self).save(*args, **kwargs)

        self._set_path()
        super(UnorderedTreeMixin, self).save(*args, **kwargs)


    def _set_path(self):

        if self.part_of:
            self.path = "%s%i/" % (self.part_of.path, self.id)
        else:
            self.path = "%i/" % self.id

    @property
    def level(self):
        "Count how far down in the tree self is"
        return unicode(self.path).count(self._sep)

    def roots(self):
        "Get all roots, nodes without parents"
        return self._default_manager.filter(part_of__isnull=True)

    def get_path(self):
        "Get all ancestors, ordered from root to self"
        return [self._default_manager.get(id=p) for p in unicode(self.path).split(self._sep) if p]

    def descendants(self):
        "Get all descendants in no particular order"
        return self._default_manager.filter(path__startswith=self.path).exclude(id=self.id)

    def parent(self):
        "Get parent of self"
        return self.part_of

    def siblings(self):
        "Get all nodes with the same parent"
        return [p for p in self.part_of.descendants() if p.level == self.level]

    def children(self):
        "Get nodes that have self as parent"
        return [p for p in self.descendants() if p.level == self.level + 1]

    def is_sibling_of(self, node):
        "Check if <node> has the same parent as self"
        return self.part_of == node.part_of

    def is_child_of(self, node):
        "Check if <node> is the parent of self"
        return self.part_of == node

    def is_root(self):
        """Check if self is a root. Roots have no parents"""
        return bool(self.part_of)

    def is_leaf(self):
        """Check if self is a leaf. Leaves have no descendants"""
        return self.descendants().count() == 0

class AbstractText(models.Model):
    "Denormalized storage of text"
    DEFAULT_TYPE = 'plaintext'
    text = models.TextField()
    text_formatted = models.TextField(editable=False)
    text_type = models.CharField(max_length=64, default=DEFAULT_TYPE)

    class Meta:
        abstract = True

    def save(self, formatters=None, *args, **kwargs):
        if self.text_type == self.DEFAULT_TYPE:
            self.text_formatted = self.text
        else:
            if formatters:
                self.text_formatted = formatters(self.text_type)
        super(AbstractText, self).save(*args, **kwargs)

class GenericForeignKeyAbstractModel(models.Model):
    """
    An abstract base class for models with one GenericForeignKey
    """

    # Content-object field
    content_type = models.ForeignKey(ContentType, verbose_name=_('content type'), related_name="content_type_set_for_%(class)s")
    object_pk = models.TextField(_('object ID'))
    content_object = generic.GenericForeignKey(ct_field="content_type", fk_field="object_pk")

    class Meta:
        abstract = True



from django.contrib import admin

from nano.comments.models import Comment

class CommentAdmin(admin.ModelAdmin):
    model = Comment
    ordering = ('added',)
    list_display = ('content_type', 'object_pk', 'comment', 'user', 'path') 
    list_filter = ('content_type',)
    date_hierarchy = 'added'
admin.site.register(Comment, CommentAdmin)




import random
import os

STORAGE_PATH = '/tmp'
FILENAME_PATTERN = 'unique_key-%s.list'
NUMERALS = '0123456789abcdefghijklmnopqrstuvwxyz'
BASE = len(NUMERALS)

def to_base(num, base, numerals=NUMERALS):
    """Convert <num> to <base> using the symbols in <numerals>"""

    assert int(num)
    assert int(base)
    if len(numerals) < base < 1:
        raise ValueError("str_base: base must be between 1 and %i" % len(numerals))

    if num == 0:
        return '0'

    if num < 0:
        sign = '-'
        num = -num
    else:
        sign = ''

    if base == 1:
        return sign + ('1'*num)

    result = ''
    while num:
        result = numerals[num % (base)] + result
        num //= base

    return sign + result

def baseNgenerator(base=10, keylength=5, step=1):
    """Generate keys of base <base> and length <keylength>"""

    assert int(base) <= BASE
    assert int(keylength) > 0
    while 1:
        yield to_base(random.randrange(base**(keylength-1), base**keylength), base)

def generate_keys(generator, amount=50):
    """Generate <amount> keys with <generator>"""

    keys = set(generator.next() for i in range(amount))
    return tuple(keys)




from django.db import models

class Link(models.Model):
    uri = models.URLField(unique=True)   
    last_checked = models.DateTimeField(blank=True, null=True)
    status = models.IntegerField(blank=True, null=True)

    class Meta:
        db_table = 'nano_link_link'

    def __unicode__(self):
        return self.uri

    def check(self):
        pass



from django import template
from django.template.loader import render_to_string
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import smart_unicode

from nano import comments
from nano.comments.models import * 

register = template.Library()

@register.inclusion_tag('nano/comments/comment_list_frag.html')
def show_comments(obj):
    contenttype = ContentType.objects.get_for_model(obj)
    comments = Comment.objects.filter(object_pk=str(obj.pk), content_type=contenttype)
    return {'comments': comments}

@register.inclusion_tag('nano/comments/comment_tree_frag.html')
def show_comments_tree(obj):
    contenttype = ContentType.objects.get_for_model(obj)
    comments = Comment.tree.roots().filter(object_pk=str(obj.pk), content_type=contenttype)
    return {'comments': comments}

@register.inclusion_tag('nano/comments/comment_tree_node_frag.html')
def show_comments_subtree(subtree):
    comments = subtree.children()
    return {'comments': comments}

@register.simple_tag
def show_comment_count(obj):
    contenttype = ContentType.objects.get_for_model(obj)
    comments = Comment.objects.filter(object_pk=str(obj.pk), content_type=contenttype).count()
    return comments

@register.filter
def comment_count(obj):
    contenttype = ContentType.objects.get_for_model(obj)
    comments = Comment.objects.filter(object_pk=str(obj.pk), content_type=contenttype).count()
    return comments



#!/usr/bin/env python

from distutils.core import setup
from setuptools import find_packages

README_FILE = open('README.rst')
try:
    long_description = README_FILE.read()
finally:
    README_FILE.close()

exclude = [
]

packages=(
        'nano',
        'nano.activation', 
        'nano.badge', 
        'nano.blog',
        'nano.chunk',
        'nano.comments',
        'nano.countries',
        'nano.faq',
        'nano.link',
        'nano.mark',
        'nano.privmsg',
        'nano.tools',
        'nano.user',
)

setup(name='nano',
        version='0.8.1',
        packages=find_packages(exclude=exclude),
        include_package_data=True,
        zip_safe=False,
        platforms=['any'],
        description='Does less! Loosely coupled mini-apps for django.',
        author_email='kaleissin@gmail.com',
        author='kaleissin',
        long_description=long_description,
        url='https://github.com/kaleissin/django-nano',
        classifiers=[
                'Development Status :: 4 - Beta',
                'Environment :: Web Environment',
                'Framework :: Django',
                'Intended Audience :: Developers',
                'License :: OSI Approved :: MIT License',
                'Operating System :: OS Independent',
                'Programming Language :: Python',
                'Topic :: Software Development :: Libraries :: Application Frameworks',
                'Topic :: Software Development :: Libraries :: Python Modules',
        ]
)



from django.conf import settings

# Optional support for django-tagging
TAGGING = None
try:
    if 'tagging' in settings.INSTALLED_APPS:
        import tagging as TAGGING
except ImportError:
    TAGGING = False

# Optional support for django-taggit
TAGGIT = None
try:
    if 'taggit' in settings.INSTALLED_APPS:
        import taggit as TAGGIT
except ImportError:
    TAGGIT = False

if TAGGIT:
    NANO_BLOG_TAGS = TAGGIT
elif TAGGING:
    NANO_BLOG_TAGS = TAGGING
else:
    NANO_BLOG_TAGS = None

# Explicit is better than implicit
if not getattr(settings, 'NANO_BLOG_USE_TAGS', False):
    NANO_BLOG_TAGS = False

NANO_BLOG_SPECIAL_TAGS = getattr(settings, 'NANO_BLOG_SPECIAL_TAGS', ('pinned',))



# -*- coding: UTF-8 -*-
from django.utils.timezone import now as tznow
from django.db import models
from django.contrib.auth.models import User

from nano.tools.models import AbstractText

class PMManager(models.Manager):

    def sent(self, user):
        """Limit to PMs sent by <user>"""

        return self.get_queryset().filter(
                sender=user, 
                sender_deleted=False)

    def archived(self, user):
        """Limit to PMs received and archived by <user>"""

        return self.get_queryset().filter(
                recipient=user,
                recipient_deleted=False,
                recipient_archived=True
                )

    def received(self, user):
        """Limit to PMs received by <user>"""

        return self.get_queryset().filter(
                recipient=user,
                recipient_archived=False, 
                recipient_deleted=False
                )

class PM(AbstractText):
    subject = models.CharField(max_length=64, blank=True, default='')
    sent = models.DateTimeField(default=tznow, editable=False)
    sender = models.ForeignKey(User, related_name='pms_sent')
    sender_deleted = models.BooleanField(default=False)
    recipient = models.ForeignKey(User, related_name='pms_received')
    recipient_archived = models.BooleanField(default=False)
    recipient_deleted = models.BooleanField(default=False)

    objects = PMManager()

    #assert False, 'tii'

    class Meta:
        db_table = 'nano_privmsg_pm'

    def __unicode__(self):
        if self.subject:
            return self.subject
        else:
            return self.content[:64]

    def save(self, *args, **kwargs):
        if not self.subject:
            snippet = self.text[:64]
            ls = len(snippet)
            if ls == 64 and ls < len(self.text):
                snippet = snippet[:-1] + u'…'
            self.subject = snippet
        super(PM, self).save(*args, **kwargs)

    def delete(self):
        if self.is_deleted():
            super(PM, self).delete()

#     @models.permalink
#     def get_absolute_url(self):
#         return ('show_pms', (), {'msgid': self.id, 'uid':}
#         )

    def is_deleted(self):
        if self.sender_deleted and self.recipient_deleted:
            return True
        if (self.sender_deleted or self.recipient_deleted) and self.sender == self.recipient:
            return True
        return False




from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.core.urlresolvers import reverse

import logging
_LOG = logging.getLogger(__name__)

from nano.tools import pop_error
from nano.privmsg.models import PM
from nano.privmsg.forms import *

def get_user(request, **kwargs):
    username = kwargs.get(u'username', None) or request.REQUEST.get(u'username', None)
    try:
        return User.objects.get(username=username)
    except User.DoesNotExist:
        pass
    user = kwargs.get(u'user', None) or request.REQUEST.get(u'user', None)
    object_id = kwargs.get(u'object_id', None) or request.REQUEST.get(u'object_id', None)
    uid = filter(None, (user, object_id))[0]
    try:
        return User.objects.get(id=int(uid))
    except User.DoesNotExist:
        return None

def _archive(user, recipient, msgids):
    if recipient != user:
        raise Http404
    if not msgids:
        raise Http404
    pms = PM.objects.filter(recipient=user, id__in=msgids)
    pms.update(recipient_archived=True)

@login_required
def move_to_archive(request, *args, **kwargs):
    next = request.GET.get('next', None)
    recipient = get_user(request, **kwargs)
    msgid = int(kwargs.get(u'msgid', None))
    _archive(request.user, recipient, (msgid,))
    if next:
        return HttpResponseRedirect(next)
    return HttpResponseRedirect(reverse('nano.privmsg.views.show_pm_archived',
            kwargs={'object_id': request.user.id}))

def _delete(user, msgid):
    if not msgid:
        raise Http404
    pm = PM.objects.get(id=msgid)
    if pm.recipient == user:
        pm.recipient_deleted=True
    if pm.sender == user:
        pm.sender_deleted=True
    pm.save()
    pm.delete()

@login_required
def delete(request, *args, **kwargs):
    next = request.GET.get('next', None)
    msgid = int(kwargs.get(u'msgid', None))
    _delete(request.user, msgid)
    if next:
        return HttpResponseRedirect(next)
    return HttpResponseRedirect(reverse('nano.privmsg.views.show_pm_received',
            kwargs={'object_id': request.user.id}))

@login_required
def show_pm_archived(request, *args, **kwargs):
    recipient = get_user(request, **kwargs)
    if recipient != request.user:
        raise Http404
    messages = PM.objects.archived(request.user)
    template = 'privmsg/list_archived.html'
    data = {'pms': messages,
            }
    return render(request, template, data)

@login_required
def show_pm_sent(request, *args, **kwargs):
    recipient = get_user(request, **kwargs)
    if recipient != request.user:
        raise Http404
    messages = PM.objects.sent(request.user)
    template = 'privmsg/list_sent.html'
    data = {'pms': messages,
            }
    return render(request, template, data)

@login_required
def show_pm_received(request, *args, **kwargs):
    recipient = get_user(request, **kwargs)
    if recipient != request.user:
        raise Http404
    messages = PM.objects.received(request.user)
    template = 'privmsg/list_received.html'
    data = {'pms': messages,
            }
    return render(request, template, data)

@login_required
def show_pms(request, *args, **kwargs):
    recipient = get_user(request, **kwargs)
    if recipient != request.user:
        raise Http404
    ACTIONS = {
        u'archive': PM.objects.archived,
        u'sent': PM.objects.sent,
        u'received': PM.objects.received,
    }
    actionstr = kwargs.get(u'action', None) or u'received'
    action = ACTIONS[actionstr]
    messages = action(request.user)
    if request.method == 'POST':
        msgids = request.POST.getlist(u'msgid')
        action = request.POST.get(u'submit')
        #assert False, '%s %s' % (action, msgids)
        if action == u'delete':
            _delete(request.user, msgids[0])
        elif action == u'archive':
            _archive(request.user, recipient, msgids)
    template = 'privmsg/archive.html'
    data = {'pms': messages,
            'action': actionstr,
            }
    return render(request, template, data)

@login_required
def add_pm(request, template='privmsg/add.html', *args, **kwargs):
    form = PMForm()
    recipient = get_user(request, **kwargs)
    next = request.GET.get('next', None)
    if request.method == 'POST':
        form = PMForm(data=request.POST)
        if form.is_valid():
            pm = form.save(commit=False)
            pm.sender = request.user
            pm.recipient = recipient
            pm.save()
            if next:
                return HttpResponseRedirect(next)
            else:
                return HttpResponseRedirect(reverse('nano.privmsg.views.show_pm_sent',
                        kwargs={'object_id': request.user.id}))
    data = {
            'pms': PM.objects.all(),
            'form': PMForm(),
            'recipient': recipient,
            }

    return render(request, template, data)



import logging
_LOG = logging.getLogger(__name__)

from django.http import HttpResponseRedirect, Http404
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.core.urlresolvers import reverse

from nano.activation.models import Key, ActivationKeyError
from nano.activation.forms import ActivationForm

from django.shortcuts import render

@login_required
def activate_key(request, template_name='nano/activation/activation_form.html', *args, **kwargs):
    """Activate a key"""

    data = {'form': ActivationForm()}
    if request.method == 'POST':
        form = ActivationForm(data=request.POST)
        if form.is_valid():
            key = form.cleaned_data['key'].lower()
            try:
                result = Key.objects.activate(key, request.user)
            except ActivationKeyError as msg:
                messages.error(request, msg, fail_silently=True)
                _LOG.error(msg)
                raise Http404
            else:
                msg = 'Key "%s" activated successfully' % key
                messages.success(request, msg, fail_silently=True)
                _LOG.info(msg)
                return HttpResponseRedirect(reverse('nano-activation-ok'))
    return render(request, template_name, data)
